require "rubygems"
require "bundler/setup"

require "sinatra"
require "slim"

DATADIR = ENV["DL_DATADIR"] || File.expand_path("../public/data", __FILE__)
URLROOT = ENV["RAILS_RELATIVE_URL_ROOT"]

get "/*" do |path|
  target = "#{DATADIR}/#{path}"
  return status 404 unless Dir.exist? target
  return redirect to("#{path}/") unless path == "" || path[-1] == "/"

  @path = path
  lastpath = "#{URLROOT}/"
  @breadcrumbs = [{ :name => URLROOT || "[root]", :path => lastpath.dup }]
  @breadcrumbs +=
    path.split(/\/+/).map do |b|
      lastpath << b << "/"
      { :name => b,
        :path => lastpath.dup }
    end
  @titlepath = @breadcrumbs[1..-1].map { |b| b[:name] }.join("/")
  @titlepath.prepend "/"
  @directories = []
  @files = []
  Dir.entries(target).sort.each do |file_name|
    file_name.force_encoding Encoding::UTF_8
    next if file_name.start_with? "."
    begin
      file = File.stat File.join(target, file_name)
    rescue Errno::ENOENT, Errno::ELOOP
      next
    end

    if file.directory?
      @directories << "#{file_name}/"
    else
      @files << { :name => file_name,
                  :size => file.size,
                  :mtime => file.mtime.getutc }
    end
  end
  slim :index
end

HUMAN_SIZE_UNITS = %w(octet octets ko Mo Go To Po)
def human_size(octets)
  ind = 0
  unless octets <= 1
    ind += 1
    while octets >= 1000
      octets /= 1000.0
      ind += 1
    end
  end
  format("%.3f %s", octets, HUMAN_SIZE_UNITS[ind])
end

REL_PATH_UNSAFE = Regexp.new("[^#{URI::REGEXP::PATTERN::UNRESERVED.delete "'"}:@&=+$,;/]").freeze
def e(*arg)
  URI.escape *arg, REL_PATH_UNSAFE
end
